/*
 * File:   main.c
 * Author: bartu
 *
 * Created on 3. dubna 2020, 12:17
 */

/*Program pro vyhodnocovn signlu z IRC senzoru*/


#include <xc.h>
#include "system.h"
#include <stdlib.h> 
#include <stdbool.h>
#include <stdio.h>
#include <libpic30.h>
#include <p30f4011.h>

// configurace oscilatoru 80MHz, FCY = 80/4
#pragma config FPR = XT_PLL8           
#pragma config FOS = PRI                
#pragma config FCKSMEN = CSW_FSCM_OFF

// vypnuti wachdog
#pragma config WDT = WDT_OFF

//configurace resetu
#pragma config FPWRT = PWRT_OFF
#pragma config BOREN = PBOR_OFF
#pragma config MCLRE = MCLR_EN


//#define RESOLUTION
//#define MAX_CNT_PER_REV (rozliseni * 4)   //pocet pulzu za otacku
//#define MAXSPEED 4000                     //maximalni rychlost
//#define HALFMAXSPEED (MAXSPEED>>1)        //polovicni rychlost

void InitQEI(void);

void InitTMR1(void);
void __attribute__((__interrupt__, __auto_psv__)) _T1Interrupt(void);

void InitTMR2(void);
void __attribute__((__interrupt__, __auto_psv__)) _T2Interrupt(void);

void delay_ms(unsigned int pozadovane_zpozdeni);

void init_MCU_pin();

void LCD_nibble(unsigned char nibble);
void LCD_priznak(unsigned char byte);
void LCD_data(unsigned char byte);
void LCD_init();
void LCD_nastav_poc();
void LCD_start();
void LCD_smazat(void);
void LCD_write(uint8_t *msg, uint8_t line);

uint8_t ctiBTN(uint8_t i);   


char line1[8];                           // 8 prvkov pole na 1 ?dek LCD
char line2[8];                           // 8 prvkov pole na 2 ?dek LCD

unsigned int citac_zpozdeni_ms = 0;      // pomocna promena na delay
            
int pocet_pulzu_op = 0;                  // opraveny pocet pulzu 
int Angle_POSCNT[2] = {0, 0};            // pole stara a nova hodnota POSCR
int POSCNTcopy = 0;                      // promena na nakopirovani POSCR
int pocet_pulzu_per = 0;                 // vypocet poctu pulzu za preruseni
float uhlova_rychlost = 0;               // uhlova rychlost (rad)
float uhel = 0;                          // uhel ()
float otacky_min = 0;                    // rpm 
float rozliseni_x4;                      // pulsy na oracku * vyhodnocovani X4
float rozliseni;                         // rozliseni senzoru

uint8_t posun_vodorovny_LCD = 0;         // vodorovny pohyb v menu
uint8_t posun_svisly_pulsy = 2;          // svisly pohyb v menu
char uvod = true;                        // zapnuti uvodni obrazovky

uint8_t press_button1,press_button2,
        press_button3,press_button4 = 0; // pomocne na zmacknuti buttonu
 
int main(void) {

    init_MCU_pin();                      

    LCD_RW = 0;                                            
    LCD_RS = 0;
    LCD_E = 1;

    InitTMR1();
    InitTMR2();
    InitQEI();

    LCD_init();
    LCD_nastav_poc();
    LCD_priznak(0b00001100);

    LED_R = 0;
    LED_G = 0;
    rozliseni = 1024;

    while (1) {

        rozliseni_x4 = rozliseni * 4;

        press_button1 = ctiBTN(1);
        press_button2 = ctiBTN(2);
        press_button3 = ctiBTN(3);
        press_button4 = ctiBTN(4);

        posun_vodorovny_LCD = posun_vodorovny_LCD - press_button1;
        posun_vodorovny_LCD = posun_vodorovny_LCD + press_button2;

        if ((uvod == true)) {
            LCD_write(" MERICI ", 0);
            LCD_write("ZARIZENI", 1);
            posun_vodorovny_LCD = 1;
            delay_ms(3000);
        }

        if (posun_vodorovny_LCD == 1) {
            uvod = false;
            LCD_write(" OTACKY ", 0);

            if (pocet_pulzu_op > 0) { // pravy smer
                line2[0] = '+';
            } 
            if(pocet_pulzu_op < 0) {
                line2[0] = '-';
            }
            if(pocet_pulzu_op == 0){
               line2[0] = '0'; 
            }
            line2[1] = '0' + (int) ((otacky_min / 1000));
            line2[2] = '0' + ((int) (otacky_min / 100)) % 10;
            line2[3] = '0' + ((int) (otacky_min / 10)) % 10;
            line2[4] = '0' + ((int) (otacky_min)) % 10;
            line2[5] = 'r';
            line2[6] = 'p';
            line2[7] = 'm';


            LCD_write(line2, 1);
        }


        if (posun_vodorovny_LCD == 2) {
            LCD_write("UHEL NAT", 0);

            line2[0] = '0' + (int) (uhel / 100);
            line2[1] = '0' + ((int) (uhel / 10)) % 10;
            line2[2] = '0' + ((int) (uhel)) % 10;
            line2[3] = 's';
            line2[4] = 't';
            line2[5] = '.';
            line2[6] = ' ';

            if (pocet_pulzu_op > 0) { // pravy smer
                line2[7] = 'P';
            } 
            if (pocet_pulzu_op < 0) {
                line2[7] = 'L';
            }
            if(pocet_pulzu_op == 0){
                line2[7] = '0';
            }
            LCD_write(line2, 1);

        }
        
         if (posun_vodorovny_LCD == 3) {
            POSCNT = 0;
            LCD_write("ROZLI:  ", 0);

            posun_svisly_pulsy = posun_svisly_pulsy + press_button3;
            posun_svisly_pulsy = posun_svisly_pulsy - press_button4;

            if (posun_svisly_pulsy == 1) {
                line2[0] = '5';
                line2[1] = '1';
                line2[2] = '2';
                line2[3] = ' ';
                line2[4] = ' ';
                rozliseni = 512;
            }
            if (posun_svisly_pulsy == 2) {
                line2[0] = '1';
                line2[1] = '0';
                line2[2] = '2';
                line2[3] = '4';
                line2[4] = ' ';
                rozliseni = 1024;
            }
            if (posun_svisly_pulsy == 3) {
                line2[0] = '2';
                line2[1] = '0';
                line2[2] = '4';
                line2[3] = '8';
                line2[4] = ' ';
                rozliseni = 2048;
            }
            if (posun_svisly_pulsy == 4) {
                line2[0] = '4';
                line2[1] = '0';
                line2[2] = '9';
                line2[3] = '6';
                line2[4] = ' ';
                rozliseni = 4096;
            }
            if (posun_svisly_pulsy == 5) {
                line2[0] = '8';
                line2[1] = '1';
                line2[2] = '9';
                line2[3] = '2';
                line2[4] = ' ';
                rozliseni = 8192;
            }

            line2[5] = ' ';
            line2[6] = ' ';
            line2[7] = ' ';

            if (posun_svisly_pulsy == 6) {
                posun_svisly_pulsy = 1;
            }
            if (posun_svisly_pulsy == 0) {
                posun_svisly_pulsy = 5;
            }
            LCD_write(line2, 1);
        }

        if (posun_vodorovny_LCD == 4) {
            posun_vodorovny_LCD = 1;
        }
        if (posun_vodorovny_LCD == 0) {
            posun_vodorovny_LCD = 3;
        }
        while(press_button1 == 1)
            press_button1=ctiBTN(1);
        while(press_button1 == 2)
            press_button1=ctiBTN(2);
        while(press_button1 == 3)
            press_button1=ctiBTN(3);
        while(press_button1 == 4) 
            press_button1=ctiBTN(4);
    }
}

uint8_t ctiBTN(uint8_t i) {

    uint8_t a, b;
    do {
        if (i == 1) {
            a = btn1;
            delay_ms(50);
            b = btn1;
        }
        if (i == 2) {
            a = btn2;
            delay_ms(50);
            b = btn2;
        }
        if (i == 3) {
            a = btn3;
            delay_ms(50);
            b = btn3;
        }
        if (i == 4) {
            a = btn4;
            delay_ms(50);
            b = btn4;
        }

    } while (a != b);
    return !a;
}

void init_MCU_pin(void) {
    ADCON1bits.ADON = 0; //vypinani AD prevodniku
    ADPCFG = 0xFFFF; // vsechny piny jsou digitlni

    TMR1 = 0; // nulovani citace
    T1CON = 0x8030;

    //nastaveni portu jako vstupni nebo vystupni
    TRISB = 0b00111000; // porty RB3 - RB5 vstupni
    TRISC = 0b00000000; // 
    TRISD = 0b00000000; // nataveno jako vystupni
    TRISE = 0b00000000; // nataveno jako vystupni
    TRISF = 0b00001111; // porty RF0 - RF3 vstupni

    // clearovani portu
    PORTB = 0;
    PORTC = 0;
    PORTD = 0;
    PORTE = 0;
    PORTF = 0;
}

void InitQEI(void) {
    
    // inicializace QEI 
    QEICONbits.QEIM = 0;
    QEICONbits.CNTERR = 1;
    QEICONbits.QEISIDL = 0;
    QEICONbits.SWPAB = 0;
    QEICONbits.PCDOUT = 0;
    QEICONbits.POSRES = 0;
    QEICONbits.TQCS = 0;
    QEICONbits.TQGATE = 0;
    QEICONbits.TQCKPS = 0;
    
    // inicializace digitalnio filtru
    DFLTCONbits.IMV1;
    DFLTCONbits.CEID = 1;
    DFLTCONbits.QEOUT = 1;
    DFLTCONbits.QECK = 1; // 1:32
    
    POSCNT = 0;
    QEICONbits.QEIM = 7; //X4, MAXCNT

    return;
}

void InitTMR1(void) {

    // Period = PR1 * prescaler * Tcy = 2343 * 64 * 50ns = 0,0075 s
    TMR1 = 0;
    T1CONbits.TON = 0;
    T1CONbits.TSIDL = 0;
    T1CONbits.TGATE = 0;
    T1CONbits.TCS = 0;
    T1CONbits.TCKPS = 2; //1:64 
    PR1 = 2343; //2343
    IFS0bits.T1IF = 0;
    IEC0bits.T1IE = 1;
    T1CONbits.TON = 1;
    return;
}

void __attribute__((__interrupt__, __auto_psv__)) _T1Interrupt(void) {

    IFS0bits.T1IF = 0;

    MAXCNT = rozliseni_x4; //nastaveni velikosti registru MAXCNT kdy pri porovnani se bude registre POSCNT resetovat
    POSCNTcopy = (int) POSCNT;

    Angle_POSCNT[1] = Angle_POSCNT[0];
    Angle_POSCNT[0] = POSCNTcopy; // hodnota registru POSCNT musise prepocitat na uhel
    if (rozliseni_x4 == 2048)
        uhel = (Angle_POSCNT[0] / 5.688);
    if (rozliseni_x4 == 4096)
        uhel = (Angle_POSCNT[0] / 11.377); 
    if (rozliseni_x4 == 8192)
        uhel = (Angle_POSCNT[0] / 22.755); 
    if (rozliseni_x4 == 16384)
        uhel = (Angle_POSCNT[0] / 45.511);
    if (rozliseni_x4 == 32768)
        uhel = (Angle_POSCNT[0] / 91.022); 
   

    pocet_pulzu_per = Angle_POSCNT[0] - Angle_POSCNT[1]; // vypocet kolik prislo pulzu za periodu

    if (pocet_pulzu_per >= 0) {
        if (pocet_pulzu_per > ((rozliseni_x4 / 2) - 1)) {
            pocet_pulzu_op = pocet_pulzu_per - rozliseni_x4;
        } else {
            pocet_pulzu_op = pocet_pulzu_per;

        }
    } else {
        if (pocet_pulzu_per < (-(rozliseni_x4 / 2))) {
            pocet_pulzu_op = pocet_pulzu_per + rozliseni_x4;

        } else {
            pocet_pulzu_op = pocet_pulzu_per;
        }
    }

    uhlova_rychlost = ((2.0 * 3.14159 * (float) pocet_pulzu_op) / (0.0075 * (float) rozliseni_x4)); // vypocet uhlove rychlosti (2*PI*n)/(N*T)

    if (uhlova_rychlost < 0)
        uhlova_rychlost = -uhlova_rychlost;

    otacky_min = uhlova_rychlost * 9.549; // prepocet na otacky za minutu
    return;
}

void InitTMR2(void) {
    // Period = PR2 * prescaler * Tcy = 312 * 64 * 50ns = 1ms

    TMR2 = 0;
    T2CONbits.TON = 0;
    T2CONbits.TSIDL = 0;
    T2CONbits.TGATE = 0;
    T2CONbits.TCS = 0;
    T2CONbits.TCKPS = 2;
    PR2 = 312;
    IFS0bits.T2IF = 0;
    IEC0bits.T2IE = 1;
    T2CONbits.TON = 1;
    return;
}

void __attribute__((__interrupt__, __auto_psv__)) _T2Interrupt(void) {
    IFS0bits.T2IF = 0;
    citac_zpozdeni_ms++;
}

void delay_ms(unsigned int pozadovane_zpozdeni) {
    citac_zpozdeni_ms = 0;
    while (citac_zpozdeni_ms < pozadovane_zpozdeni);
}

void LCD_write(uint8_t *msg, uint8_t line) {

    uint8_t i;
    if (line == 0) {
        LCD_priznak(0x02);

    }
    if (line == 1) {
        LCD_priznak(0xC0);
    }
    for (i = 0; i < 8; i++) {
        LCD_data(msg[i]);
    }
}

void LCD_nibble(unsigned char nibble) {
    LATD = nibble;
    delay_ms(1);
    LCD_E = 0;
    delay_ms(1);
    LCD_E = 1;
    delay_ms(2);
}

void LCD_priznak(unsigned char byte) {
    LCD_RS = 0;
    LCD_nibble(byte >> 4);
    LCD_nibble(byte & 0xF);
}

void LCD_data(unsigned char byte) {
    LCD_RS = 1;
    LCD_nibble(byte >> 4);
    LCD_nibble(byte & 0xF);
}

void LCD_init(void) {
    delay_ms(16);
    LCD_nibble(0b0011);
    delay_ms(5);
    LCD_nibble(0b0011);
    delay_ms(1);
    LCD_nibble(0b0011);
    delay_ms(5);
    LCD_nibble(0b0010);
}

void LCD_nastav_poc(void) {
    LCD_priznak(0b00101000);
    LCD_priznak(0b00001000);
    LCD_priznak(0b00000001);
    LCD_priznak(0b00000110);
    LCD_priznak(0b00001111);
}

void LCD_smazat(void) {
    LCD_priznak(0x01);
    delay_ms(2);
}